package com.fs.course.controller;

import com.alibaba.fastjson.JSON;
import com.fs.common.annotation.Log;
import com.fs.common.core.controller.BaseController;
import com.fs.common.core.domain.AjaxResult;
import com.fs.common.core.domain.R;
import com.fs.common.core.page.TableDataInfo;
import com.fs.common.enums.BusinessType;
import com.fs.common.utils.StringUtils;
import com.fs.common.utils.poi.ExcelUtil;
import com.fs.course.domain.FsUserVideo;
import com.fs.course.domain.FsVideoResource;
import com.fs.course.param.*;
import com.fs.course.service.IFsUserVideoService;
import com.fs.course.service.IFsVideoResourceService;
import com.fs.course.service.impl.TencentCloudCosService;
import com.fs.course.vo.FsUserVideoListPVO;
import com.fs.course.vo.FsUserVideoPVO;
import com.fs.his.service.IFsPackageService;
import com.fs.his.vo.FsPackageVO;
import com.fs.system.oss.CloudStorageService;
import com.fs.system.oss.OSSFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FilenameUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * 课堂视频Controller
 *
 * @author fs
 * @date 2024-05-15
 */
@RestController
@RequestMapping("/course/userVideo")
@Slf4j
public class FsUserVideoController extends BaseController
{
    @Autowired
    private IFsUserVideoService fsUserVideoService;

    @Autowired
    private TencentCloudCosService tencentCloudCosService;
    @Autowired
    private IFsPackageService fsPackageService;

    @Autowired
    private IFsVideoResourceService iFsVideoResourceService;

    /**
     * 查询课堂视频列表
     */
    @PreAuthorize("@ss.hasPermi('course:userVideo:list')")
    @GetMapping("/list")
    public TableDataInfo list(FsUserVideoParam fsUserVideo)
    {
        startPage();
        List<FsUserVideoListPVO> list = fsUserVideoService.selectFsUserVideoListPVO(fsUserVideo);
        return getDataTable(list);
    }

    /**
     * 导出课堂视频列表
     */
    @PreAuthorize("@ss.hasPermi('course:userVideo:export')")
    @Log(title = "课堂视频", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(FsUserVideo fsUserVideo)
    {
        List<FsUserVideo> list = fsUserVideoService.selectFsUserVideoList(fsUserVideo);
        ExcelUtil<FsUserVideo> util = new ExcelUtil<FsUserVideo>(FsUserVideo.class);
        return util.exportExcel(list, "课堂视频数据");
    }

    /**
     * 获取课堂视频详细信息
     */
    @PreAuthorize("@ss.hasPermi('course:userVideo:query')")
    @GetMapping(value = "/{videoId}")
    public AjaxResult getInfo(@PathVariable("videoId") String videoId)
    {
        return AjaxResult.success(fsUserVideoService.selectFsUserVideoByVideoId(videoId));
    }

    @GetMapping(value = "/getVideoDetails")
    public R getVideoDetails(@RequestParam("videoId") Long videoId)
    {
        FsUserVideoPVO videoPVO = fsUserVideoService.selectFsUserVideoPVOByVideoId(videoId);
        FsPackageVO packageVO =null;
        if (videoPVO.getProductId()!=null ) {
            packageVO = fsPackageService.selectFsPackageByPackageIdVO(videoPVO.getProductId());
        }
       return R.ok().put("data", videoPVO).put("package", packageVO);
    }

    /**
     * 新增课堂视频
     */
    @PreAuthorize("@ss.hasPermi('course:userVideo:add')")
    @Log(title = "课堂视频", businessType = BusinessType.INSERT)
    @PostMapping("/addVideo")
    public R add(@RequestBody FsUserVideoAddParam param)
    {
        return fsUserVideoService.addUserVideo(param);
    }

    /**
     * 修改课堂视频
     */
    @PreAuthorize("@ss.hasPermi('course:userVideo:edit')")
    @Log(title = "课堂视频", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody FsUserVideo fsUserVideo)
    {
        return toAjax(fsUserVideoService.updateFsUserVideo(fsUserVideo));
    }

    /**
     * 删除课堂视频
     */
    @PreAuthorize("@ss.hasPermi('course:userVideo:remove')")
    @Log(title = "课堂视频", businessType = BusinessType.DELETE)
	@DeleteMapping("/{videoIds}")
    public AjaxResult remove(@PathVariable String[] videoIds)
    {
        return toAjax(fsUserVideoService.deleteFsUserVideoByVideoIds(videoIds));
    }

    @GetMapping("/getVideoListByCourseId")
    public TableDataInfo getVideoListByCourseId(FsUserVideo fsUserVideo)
    {
        startPage();
        List<FsUserVideo> list = fsUserVideoService.selectFsUserVideoListByCourseId(fsUserVideo);
        return getDataTable(list);
    }


    @PreAuthorize("@ss.hasPermi('course:userVideo:audit')")
    @Log(title = "审核视频", businessType = BusinessType.UPDATE)
    @PostMapping("/auditVideo")
    public R auditVideo(@RequestBody FsUserVideoAuditParam param)
    {
        param.setAuditTime(new Date());
        param.setAuditBy(getUserId());
        if (param.getIsAudit()==-1){
            param.setStatus(0);
        }
        if (fsUserVideoService.auditUserVideos(param)>0){
            return R.ok("审核成功");
        }
        return R.error("审核失败");
    }

    @PreAuthorize("@ss.hasPermi('course:userVideo:putOn')")
    @Log(title = "短视频批量上架", businessType = BusinessType.UPDATE)
    @PostMapping("/putOn/{videoIds}")
    public AjaxResult putOn(@PathVariable Long[] videoIds)
    {
        return toAjax(fsUserVideoService.updateFsUserVideoIsShow(videoIds,1));
    }

    @PreAuthorize("@ss.hasPermi('course:userVideo:putOn')")
    @Log(title = "短视频批量下架", businessType = BusinessType.UPDATE)
    @PostMapping("/pullOff/{videoIds}")
    public AjaxResult pullOff(@PathVariable Long[] videoIds)
    {
        return toAjax(fsUserVideoService.updateFsUserVideoIsShow(videoIds,0));
    }

    private static final String VIDEO_UPLOAD_DIR = "C:\\fs\\uploadPath\\userVideo\\video";  // 上传目录
    private static final String FRAME_OUTPUT_DIR = "C:\\fs\\uploadPath\\userVideo\\frame";  // 输出帧的目录


    /**
     * 获取视频第一帧为封面
     * @param file
     * @return
     * @throws Exception
     */
    @PostMapping("/getThumbnail")
    public R uploadVideo(@RequestParam("file") MultipartFile file) throws Exception {
        if (file.isEmpty() || !file.getContentType().equals("video/mp4")) {
            return R.error("仅支持MP4类型");
        }

        // 保存上传的视频文件
        String videoFileName = System.currentTimeMillis() + "_" + UUID.randomUUID().toString().replaceAll("-", "").substring(0, 16);
        File videoFile = new File(VIDEO_UPLOAD_DIR, videoFileName);
        try {
            file.transferTo(videoFile);
        } catch (IOException e) {
            // 记录错误日志
            return R.error("获取封面失败");
        }

        // 提取视频第一帧
        String frameFileName = FilenameUtils.removeExtension(videoFileName) + "_frame.jpg";
        File frameFile = new File(FRAME_OUTPUT_DIR, frameFileName);
        try {
            extractFirstFrame(videoFile.getAbsolutePath(), frameFile.getAbsolutePath());
        } catch (Exception e) {
            // 记录错误日志
            return R.error("提取第一帧失败");
        }

        // 上传第一帧图片到 COS
        String suffix = ".jpg";
        CloudStorageService storage = OSSFactory.build();
        String url;
        try (InputStream inputStream = new FileInputStream(frameFile)) {
            url = storage.uploadSuffix(inputStream, suffix);
        }  catch (Exception e) {
            // 记录错误日志
            log.error("上传封面图失败:{}",e.getMessage(),e);
            return R.error("上传封面图失败");
        } finally {
            // 清理临时文件
            videoFile.delete();
            frameFile.delete();
        }

        // 返回第一帧图片的 COS URL
        return R.ok().put("url", url);
    }

    // 提取视频第一帧的方法
    private void extractFirstFrame(String videoPath, String outputImagePath) throws IOException, InterruptedException {
        String[] command = {
                "ffmpeg",
                "-i", videoPath,
//                "-ss", "00:00:01.000", // 截取1秒处的帧
                "-frames:v", "1",
                "-f", "image2",
                "-update", "1",
                "-q:v", "10", // 质量，1为最好，31为最差
                outputImagePath
        };
        ProcessBuilder processBuilder = new ProcessBuilder(command);
        processBuilder.redirectErrorStream(true); // 将标准错误和标准输出合并
        Process process = processBuilder.start();
//        process.waitFor();

        // 处理进程的标准输出和错误流
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()))) {
            String line;
            while ((line = reader.readLine()) != null) {
                System.out.println(line); // 输出到控制台或日志
            }
        }

        int exitCode = process.waitFor();
        if (exitCode != 0) {
            throw new RuntimeException("FFmpeg 执行失败，退出代码：" + exitCode);
        }
    }

    @PostMapping("/updateUrl")
    public R updateUrl()
    {
        log.info("开始更新URL");
        try {
            fsUserVideoService.updateVideoUrl();
            log.info("更新URL成功完成");
        } catch (Exception e) {
            log.error("开始更新URL执行失败", e);
        }
        return R.ok();
    }

    /**
     * 点播编排转码
     * @return
     */
    @PostMapping("/videoTranscode")
    public R videoTranscode(@RequestBody String rawBody) {
        try {
            // 解析输入JSON
            VideoTranscodeParam videoTranscodeParam = JSON.parseObject(rawBody, VideoTranscodeParam.class);
            if (videoTranscodeParam == null || videoTranscodeParam.getScheduleTaskEvent() == null) {
                log.error("请求体格式无效或缺少scheduleTaskEvent信息");
                return R.error("请求格式无效");
            }

            // 检查错误码
            VideoScheduleTaskEventParam event = videoTranscodeParam.getScheduleTaskEvent();
            if (!Integer.valueOf(0).equals(event.getErrCode())) {
                log.warn("收到转码事件非零错误码: {}", event.getErrCode());
                return R.ok(); // 仍然返回ok确认已处理通知
            }

            // 验证输入路径
            if (event.getInputInfo() == null || event.getInputInfo().getCosInputInfo() == null) {
                log.error("转码事件中缺少输入信息");
                return R.error("缺少输入信息");
            }

            String inputPaths = event.getInputInfo().getCosInputInfo().getObject();
            if (StringUtils.isBlank(inputPaths)) {
                log.error("转码事件中的输入路径为空");
                return R.error("输入路径为空");
            }

            // 移除可能的前导斜杠
            String inputPath = inputPaths.startsWith("/") ? inputPaths.substring(1) : inputPaths;

            // 查找视频资源
            FsVideoResource video = iFsVideoResourceService.selectByFileKey(inputPath);
            if (video == null) {
                log.warn("未找到文件键对应的视频资源: {}", inputPath);
                return R.ok();
            }

            // 验证输出路径
            if (event.getActivityResultSet() == null || event.getActivityResultSet().isEmpty()) {
                log.error("转码事件中缺少活动结果集");
                return R.error("缺少活动结果");
            }

            VideoActivityResItemParam activityResItem = event.getActivityResultSet().get(0).getActivityResItem();
            if (activityResItem == null || activityResItem.getTranscodeTask() == null ||
                    activityResItem.getTranscodeTask().getOutput() == null) {
                log.error("转码任务信息不完整");
                return R.error("转码任务不完整");
            }

            String outputPath = activityResItem.getTranscodeTask().getOutput().getPath();
            if (StringUtils.isBlank(outputPath)) {
                log.error("转码事件中的输出路径为空");
                return R.error("输出路径为空");
            }

            // 移除可能的前导斜杠
            String transcodeFileKey = outputPath.startsWith("/") ? outputPath.substring(1) : outputPath;

            // 提交转码作业
            log.info("开始提交转码作业，输入路径: {}, 输出路径: {}", inputPaths, outputPath);
            tencentCloudCosService.submitTranscodeJob(inputPaths, outputPath);

            // 更新视频资源
            FsVideoResource videoMap = new FsVideoResource();
            videoMap.setId(video.getId());
            videoMap.setIsTranscode(1);
            videoMap.setLine1(video.getLine1().replace(inputPath,transcodeFileKey));
            videoMap.setTranscodeFileKey(transcodeFileKey);
            iFsVideoResourceService.updateById(videoMap);
            log.info("成功更新视频转码状态，视频ID: {}", video.getId());

            return R.ok();
        } catch (Exception e) {
            log.error("处理视频转码请求时发生异常", e);
            return R.error("服务器内部错误");
        }
    }
}
