package com.fs.hospital.controller.hospital;

import com.fs.common.annotation.Log;
import com.fs.common.constant.UserConstants;
import com.fs.common.core.controller.BaseController;
import com.fs.common.core.domain.AjaxResult;
import com.fs.common.enums.BusinessType;
import com.fs.common.utils.ServletUtils;
import com.fs.common.utils.StringUtils;
import com.fs.hospital.domain.HospitalDept;
import com.fs.hospital.service.IHospitalDeptService;
import com.fs.framework.security.LoginUser;
import com.fs.framework.security.SecurityUtils;
import com.fs.framework.service.TokenService;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Iterator;
import java.util.List;

/**
 * 部门信息
 *

 */
@RestController
@RequestMapping("/hospital/dept")
public class HospitalDeptController extends BaseController
{
    @Autowired
    private IHospitalDeptService deptService;
    @Autowired
    private TokenService tokenService;
    /**
     * 获取部门列表
     */
    @GetMapping("/list")
    public AjaxResult list(HospitalDept dept)
    {
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        dept.setHospitalId(loginUser.getHospital().getHospitalId());
        List<HospitalDept> depts = deptService.selectHospitalDeptList(dept);
        return AjaxResult.success(depts);
    }

    /**
     * 查询部门列表（排除节点）
     */
    @PreAuthorize("@ss.hasPermi('hospital:dept:list')")
    @GetMapping("/list/exclude/{deptId}")
    public AjaxResult excludeChild(@PathVariable(value = "deptId", required = false) Long deptId)
    {
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        HospitalDept dept=new HospitalDept();
        dept.setHospitalId(loginUser.getHospital().getHospitalId());
        List<HospitalDept> depts = deptService.selectHospitalDeptList(dept);
        Iterator<HospitalDept> it = depts.iterator();
        while (it.hasNext())
        {
            HospitalDept d = (HospitalDept) it.next();
            if (d.getDeptId().intValue() == deptId
                    || ArrayUtils.contains(StringUtils.split(d.getAncestors(), ","), deptId + ""))
            {
                it.remove();
            }
        }
        return AjaxResult.success(depts);
    }

    /**
     * 根据部门编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('hospital:dept:query')")
    @GetMapping(value = "/{deptId}")
    public AjaxResult getInfo(@PathVariable Long deptId)
    {
        return AjaxResult.success(deptService.selectHospitalDeptByDeptId(deptId));
    }

    /**
     * 获取部门下拉树列表
     */
    @GetMapping("/treeselect")
    public AjaxResult treeselect(HospitalDept dept)
    {
        System.out.println("查询部门");
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        dept.setHospitalId(loginUser.getHospital().getHospitalId());
        dept.setStatus("0");
        List<HospitalDept> depts = deptService.selectHospitalDeptList(dept);
        return AjaxResult.success(deptService.buildDeptTreeSelect(depts));
    }
    /**
     * 加载对应角色部门列表树
     */
    @GetMapping(value = "/roleDeptTreeselect/{roleId}")
    public AjaxResult roleDeptTreeselect(@PathVariable("roleId") Long roleId)
    {
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        HospitalDept dept=new HospitalDept();
        dept.setHospitalId(loginUser.getHospital().getHospitalId());
        dept.setStatus("0");
        List<HospitalDept> depts = deptService.selectHospitalDeptList(dept);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("checkedKeys", deptService.selectDeptListByRoleId(roleId));
        ajax.put("depts", deptService.buildDeptTreeSelect(depts));
        return ajax;
    }

    /**
     * 新增部门
     */
    @PreAuthorize("@ss.hasPermi('hospital:dept:add')")
    @Log(title = "部门管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody HospitalDept dept)
    {
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        dept.setHospitalId(loginUser.getHospital().getHospitalId());
        if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept)))
        {
            return AjaxResult.error("新增部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        dept.setCreateBy(SecurityUtils.getUsername());
        return toAjax(deptService.insertHospitalDept(dept));
    }

    /**
     * 修改部门
     */
    @PreAuthorize("@ss.hasPermi('hospital:dept:edit')")
    @Log(title = "部门管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody HospitalDept dept)
    {
        LoginUser loginUser = tokenService.getLoginUser(ServletUtils.getRequest());
        dept.setHospitalId(loginUser.getHospital().getHospitalId());
        if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept)))
        {
            return AjaxResult.error("修改部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        else if (dept.getParentId().equals(dept.getDeptId()))
        {
            return AjaxResult.error("修改部门'" + dept.getDeptName() + "'失败，上级部门不能是自己");
        }
        else if (StringUtils.equals(UserConstants.DEPT_DISABLE, dept.getStatus())
                && deptService.selectNormalChildrenDeptById(dept.getDeptId()) > 0)
        {
            return AjaxResult.error("该部门包含未停用的子部门！");
        }
        dept.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(deptService.updateHospitalDept(dept));
    }

    /**
     * 删除部门
     */
    @PreAuthorize("@ss.hasPermi('hospital:dept:remove')")
    @Log(title = "部门管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{deptId}")
    public AjaxResult remove(@PathVariable Long deptId)
    {
        if (deptService.hasChildByDeptId(deptId)>0)
        {
            return AjaxResult.error("存在下级部门,不允许删除");
        }
        if (deptService.checkDeptExistUser(deptId)>0)
        {
            return AjaxResult.error("部门存在用户,不允许删除");
        }
        return toAjax(deptService.deleteHospitalDeptByDeptId(deptId));
    }





}
