package com.fs.app.redis;//package com.fs.app.redis;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import com.fs.common.constant.FsConstants;
import com.fs.common.core.redis.RedisCache;
import com.fs.his.config.FsSysConfig;
import com.fs.his.utils.ConfigUtil;
import com.fs.qw.domain.QwCompany;
import com.fs.qw.domain.QwUser;
import com.fs.qw.mapper.QwCompanyMapper;
import com.fs.qw.mapper.QwUserMapper;
import com.fs.qw.param.QwLoginParam;
import com.fs.qw.service.IQwUserService;
import com.fs.qwApi.param.QwSendMsgParam;
import com.fs.qwApi.service.QwApiService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.listener.KeyExpirationEventMessageListener;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;
import java.util.concurrent.TimeUnit;

@Component
public class RedisKeyExpirationListener extends KeyExpirationEventMessageListener {

    private static final Logger log = LoggerFactory.getLogger(RedisKeyExpirationListener.class);

    public RedisKeyExpirationListener(RedisMessageListenerContainer listenerContainer) {
        super(listenerContainer);
    }

    @Autowired
    private IQwUserService qwUserService;
    @Autowired
    private ConfigUtil configUtil;

    @Autowired
    private StringRedisTemplate redisTemplate;  // 使用 RedisTemplate 进行分布式锁

    @Autowired
    QwUserMapper qwUserMapper;

    @Autowired
    QwCompanyMapper qwCompanyMapper;

    @Autowired
    QwApiService qwApiService;

    /**
     * 针对redis数据失效事件，进行数据处理
     * @param message
     * @param pattern
     */
    @Override
    public void onMessage(Message message, byte[] pattern) {
        String channel = new String(message.getChannel(), StandardCharsets.UTF_8);
        //过期的key
        String key = new String(message.getBody(),StandardCharsets.UTF_8);

        //判断是否是appKey失效
        if(key.contains(FsConstants.REDIS_QW_appKey_Active)) {

            log.info("监听Qw过期appKey-redis过期：pattern={},channel={},key={}",new String(pattern),channel,key);

            String lockKey = "lockAppKey:" + key;  // 分布式锁的 key

            // 尝试获取锁，设置 30 秒自动过期，防止死锁
            Boolean lockAcquired = redisTemplate.opsForValue().setIfAbsent(lockKey, "1", 30, TimeUnit.SECONDS);

            if (Boolean.TRUE.equals(lockAcquired)) {  // 只有一个实例会成功获取锁
                try {
                    String[] parts = key.split(":");

                    if (parts.length == 3) {
                        String corpId = parts[1].trim();
                        String qwUserId = parts[2].trim();

                        String appKey = qwUserService.selectQwUserByQwUserIdAndCorId(qwUserId, corpId);

                        String msg="<font color=\"warning\">您的【云联融智】助手已退出，企业微信【侧边栏】有异常，发送已停止，请注意查看</font>";
                        sendQwMsg(appKey,msg);

                        log.info("监听Qw过期appKey-redis过期：pattern={},channel={},key={}",new String(pattern),channel,key);
                    } else {
                        System.out.println("监听appKey失效！");
                    }
                } finally {
                    // 释放锁，避免影响后续任务
                    redisTemplate.delete(lockKey);
                }
            } else {
                log.info("另一个实例已经处理了 key={}，当前实例跳过", key);
            }

        }
    }

    /**
     * 给企业微信的应用发送消息
     */
    private void sendQwMsg(String appKey,String msg){
        QwUser qwUserByAppKey = qwUserMapper.selectQwUserByAppKey(appKey);

        QwSendMsgParam sendMsgParam = new QwSendMsgParam();
        QwCompany qwCompany = qwCompanyMapper.selectQwCompanyByCorpId(qwUserByAppKey.getCorpId());
        sendMsgParam.setAgentid(Integer.parseInt(qwCompany.getServerAgentId().trim()));
        sendMsgParam.setTouser(qwUserByAppKey.getQwUserId());

        QwSendMsgParam.Markdown markdown = new QwSendMsgParam.Markdown();
        markdown.setContent(msg);

        sendMsgParam.setMarkdown(markdown);
        sendMsgParam.setMsgtype("markdown");
        qwApiService.sendMsg(sendMsgParam, qwCompany.getCorpId());

    }
}
