package com.fs.course.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fs.common.exception.ServiceException;
import com.fs.course.domain.FsUserCoursePeriod;
import com.fs.course.domain.FsUserCoursePeriodDays;
import com.fs.course.mapper.FsUserCoursePeriodDaysMapper;
import com.fs.course.mapper.FsUserCoursePeriodMapper;
import com.fs.course.service.IFsUserCoursePeriodService;
import com.fs.course.vo.FsUserCoursePeriodVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 会员营期Service业务层处理
 *
 * @author fs
 * @date 2025-04-15
 */
@Service
public class FsUserCoursePeriodServiceImpl implements IFsUserCoursePeriodService
{
    @Autowired
    private FsUserCoursePeriodMapper fsUserCoursePeriodMapper;
    @Resource
    private FsUserCoursePeriodDaysMapper fsUserCoursePeriodDaysMapper;

    /**
     * 查询会员营期
     *
     * @param periodId 会员营期ID
     * @return 会员营期
     */
    @Override
    public FsUserCoursePeriod selectFsUserCoursePeriodById(Long periodId)
    {
        return fsUserCoursePeriodMapper.selectFsUserCoursePeriodById(periodId);
    }

    /**
     * 查询会员营期列表
     *
     * @param fsUserCoursePeriod 会员营期
     * @return 会员营期
     */
    @Override
    public List<FsUserCoursePeriod> selectFsUserCoursePeriodList(FsUserCoursePeriod fsUserCoursePeriod)
    {
        return fsUserCoursePeriodMapper.selectFsUserCoursePeriodList(fsUserCoursePeriod);
    }

    /**
     * 新增会员营期
     *
     * @param fsUserCoursePeriod 会员营期
     * @return 结果
     */
    @Override
    public int insertFsUserCoursePeriod(FsUserCoursePeriod fsUserCoursePeriod)
    {
        fsUserCoursePeriod.setCreateTime(LocalDateTime.now());
        // 设置营期状态
        if(LocalDate.now().isBefore(fsUserCoursePeriod.getPeriodStartingTime())){
            fsUserCoursePeriod.setPeriodStatus(1L);
        } else if(LocalDate.now().isAfter(fsUserCoursePeriod.getPeriodEndTime())){
            fsUserCoursePeriod.setPeriodStatus(3L);
        } else{
            fsUserCoursePeriod.setPeriodStatus(2L);
        }
        return fsUserCoursePeriodMapper.insertFsUserCoursePeriod(fsUserCoursePeriod);
    }

    /**
     * 修改会员营期
     *
     * @param fsUserCoursePeriod 会员营期
     * @return 结果
     */
    @Override
    public int updateFsUserCoursePeriod(FsUserCoursePeriod fsUserCoursePeriod) {
        // 1. 查询原始数据
        fsUserCoursePeriod.setUpdateTime(LocalDateTime.now());

        FsUserCoursePeriod fsUserCoursePeriod1 = fsUserCoursePeriodMapper.selectFsUserCoursePeriodById(fsUserCoursePeriod.getPeriodId());
        int flag = fsUserCoursePeriodMapper.updateFsUserCoursePeriod(fsUserCoursePeriod);

        // 2. 判定是否变更过开始时间（periodStartingTime）
        if (fsUserCoursePeriod1.getPeriodStartingTime().equals(fsUserCoursePeriod.getPeriodStartingTime())) {
            // 如果开始时间没有变化，直接返回
            return flag;
        }

        // 3. 计算课程时间差（以天数为单位）
        long daysDifference = ChronoUnit.DAYS.between(fsUserCoursePeriod1.getPeriodStartingTime(), fsUserCoursePeriod.getPeriodStartingTime());

        // 4. 获取课程天数并进行时间调整
        FsUserCoursePeriodDays fsUserCoursePeriodDays = new FsUserCoursePeriodDays();
        fsUserCoursePeriodDays.setPeriodId(fsUserCoursePeriod1.getPeriodId());
        List<FsUserCoursePeriodDays> list = fsUserCoursePeriodDaysMapper.selectFsUserCoursePeriodDaysList(fsUserCoursePeriodDays);

        // 对每个课程日期进行时间增加
        for (FsUserCoursePeriodDays coursePeriodDays : list) {
            // 将课程的日期按照差值天数进行调整
            LocalDate adjustedDate = coursePeriodDays.getDayDate().plusDays(daysDifference);
            LocalDateTime startDateTime = coursePeriodDays.getStartDateTime().plusDays(daysDifference);
            LocalDateTime endDateTime = coursePeriodDays.getEndDateTime().plusDays(daysDifference);
            LocalDateTime lastJsonTime = coursePeriodDays.getLastJoinTime().plusDays(daysDifference);
            coursePeriodDays.setDayDate(adjustedDate);
            coursePeriodDays.setStartDateTime(startDateTime);
            coursePeriodDays.setEndDateTime(endDateTime);
            coursePeriodDays.setLastJoinTime(lastJsonTime);
            fsUserCoursePeriodDaysMapper.updateFsUserCoursePeriodDays(coursePeriodDays); // 更新数据库中的课程日期
        }

        return flag;
    }

    /**
     * 批量删除会员营期
     *
     * @param periodIds 需要删除的会员营期ID
     * @return 结果
     */
    @Override
    public int deleteFsUserCoursePeriodByIds(Long[] periodIds)
    {
        if (checkPeriodStatus(periodIds)) {
            throw new ServiceException("存在进行中或已结束的营期，不允许删除");
        }
        return fsUserCoursePeriodMapper.updateBatchDelFlag(periodIds,1);
    }
    private boolean checkPeriodStatus(Long[] ids) {
        LocalDate currentDate = LocalDate.now(); // 2025-06-30
        Set<Long> idSet = Arrays.stream(ids).collect(Collectors.toSet());
        List<FsUserCoursePeriod> list = fsUserCoursePeriodMapper.selectFsUserCoursePeriodsByIds(idSet);

        List<FsUserCoursePeriod> filteredList = list.stream()
                .filter(period -> {
                    LocalDate startTime = period.getPeriodStartingTime();
                    LocalDate endTime = period.getPeriodEndTime();

                    boolean isActive = startTime != null && !startTime.isAfter(currentDate)
                            && (endTime == null || !endTime.isBefore(currentDate));

                    return isActive;
                })
                .collect(Collectors.toList());

        return !filteredList.isEmpty();
    }

    /**
     * 删除会员营期信息
     *
     * @param periodId 会员营期ID
     * @return 结果
     */
    @Override
    public int deleteFsUserCoursePeriodById(Long periodId)
    {
        return fsUserCoursePeriodMapper.deleteFsUserCoursePeriodById(periodId);
    }

    @Override
    public List<FsUserCoursePeriodVO> selectFsUserCoursePeriodPage(FsUserCoursePeriod fsUserCoursePeriod) {
        return fsUserCoursePeriodMapper.selectFsUserCoursePeriodPage(fsUserCoursePeriod);
    }

    /**
     * 结束营期
     * @param id    营期ID
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void closePeriod(Long id) {
        // 修改营期结束时间
        FsUserCoursePeriod fsUserCoursePeriod = fsUserCoursePeriodMapper.selectFsUserCoursePeriodById(id);
        if (Objects.isNull(fsUserCoursePeriod)) {
            throw new ServiceException("结束营期失败：营期不存在");
        }

        if (fsUserCoursePeriod.getPeriodStatus() == 1) {
            throw new ServiceException("结束营期失败：营期未开始不能进行结束操作！");
        }

        if (fsUserCoursePeriod.getPeriodStatus() == 3) {
            throw new ServiceException("结束营期失败：营期已结束不能进行结束操作！");
        }

        fsUserCoursePeriod.setPeriodEndTime(LocalDate.now());
        fsUserCoursePeriodMapper.updateFsUserCoursePeriod(fsUserCoursePeriod);

        // 修改大于当前时间的课程状态
        Wrapper<FsUserCoursePeriodDays> updateWrapper = Wrappers.<FsUserCoursePeriodDays>lambdaUpdate()
                .set(FsUserCoursePeriodDays::getStatus, 2)
                .eq(FsUserCoursePeriodDays::getPeriodId, fsUserCoursePeriod.getPeriodId())
                .eq(FsUserCoursePeriodDays::getStatus, 0)
                .gt(FsUserCoursePeriodDays::getDayDate, LocalDate.now());
        fsUserCoursePeriodDaysMapper.update(null, updateWrapper);
    }

    /**
     * 更新营期状态
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void changePeriodStatus() {
        // 开营
        fsUserCoursePeriodMapper.startPeriod(LocalDate.now());

        // 关营
        fsUserCoursePeriodMapper.endPeriod(LocalDate.now());
    }
}
